/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	* This work (DesQLayout) was inspired by Qt exmaples
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

/*
	*
	* About DesQLayout - https://github.com/VaysseB/SmartVerticalFlowLayout
	*
	* DesQLayout is a highly customized layout meant to be used in the DesQ Project.
	* DesQLayout is a dynamic layout that switches from horizontal to vertical layout mode
	* if the available width goes below a particular value (default: 300 px).
	*
*/

#pragma once

#include <DesQUI.hpp>

QT_BEGIN_NAMESPACE

class DESQUI_DLLSPEC DesQLayout : public QLayout {
	Q_OBJECT

	Q_PROPERTY( int horizontalSpacing READ horizontalSpacing WRITE setHorizontalSpacing )
	Q_PROPERTY( int verticalSpacing READ verticalSpacing WRITE setVerticalSpacing )
	Q_PROPERTY( Qt::Alignment alignment READ alignment WRITE setAlignment )
	Q_PROPERTY( int maxRowCount READ maxRowCount WRITE setMaxRowCount )

	public:
		explicit DesQLayout( QWidget *parent = 0 );
		~DesQLayout( );

		int count( ) const;
		void addItem( QLayoutItem *item );
		QLayoutItem *itemAt( int index ) const;
		QLayoutItem *takeAt( int index );

		int horizontalSpacing( ) const;
		int verticalSpacing( ) const;

		bool hasHeightForWidth( ) const;
		int heightForWidth( int width ) const;

		int maxRowCount( ) const;

		/**
		 * @brief Return with no expanding direction if aligment isn't justify nor 0.
		 * @return Qt::Horizontal or 0
		 */
		Qt::Orientations expandingDirections( ) const;

		QSize sizeHint( ) const;
		QSize minimumSize( ) const;
		void setGeometry( const QRect &rect );

	public slots:
		void setHorizontalSpacing( int space );
		void setVerticalSpacing( int space );

		void setMaxRowCount( int count );

		/**
		 * @brief Change alignment of all items.
		 * You must call DesQLayout::updateLayout( ) if called as
		 * QLayout* or QLayoutItem* because the function isn't virtual
		 *
		 * @param align Items alignment ( 0, Justify, Left, Center, Right )
		 */
		void setAlignment( Qt::Alignment align );

		/**
		 * @brief Force layout to reprocess all items positions.
		 */
		void updateLayout( );

	private:
		void doLayout( const QRect &rect, bool dummy ) const;
		int smartSpacing( QStyle::PixelMetric pm ) const;

	private:
		QList<QLayoutItem *> m_items;
		int m_hSpace = -1;
		int m_vSpace = -1;

		int m_maxRowCount = -1;

		mutable QList<QList<QLayoutItem*> > m_structure;
		mutable QRect m_structureGeometry;
		mutable bool m_isLayoutModified = false;
};


QT_END_NAMESPACE
