/*
	*
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <DesQComboBox.hpp>

DesQComboBox::DesQComboBox( QString label, QStringList items, bool compact, QWidget *parent ) : DesQWidget( parent ) {

	titleLbl = new QLabel( label );

	edit = new QComboBox( this );
	edit->addItems( items );
	edit->setMinimumHeight( 24 );

	if ( label.contains( "&" ) )
		titleLbl->setBuddy( edit );

	edit->setStyleSheet(
		"QComboBox {"
		"	border: none;"
		"	border-bottom: 2px solid gray;"
		"	background: transparent;"
		"}"
		"QComboBox:focus {"
		"	border-bottom: 2px solid palette(Highlight);"
		"}"
		"QComboBox::drop-down {"
		"	border: none;"
		"}"
		"QComboBox::down-arrow {"
		"	image: url('/usr/share/icons/breeze-dark/actions/16/arrow-down.svg');"
		"	height: 8px;"
		"	width: 8px;"
		"}"
	);

	QBoxLayout *lyt;
	if ( compact )
		lyt = new QVBoxLayout();

	else
		lyt = new QHBoxLayout();

	lyt->setContentsMargins( QMargins() );
	lyt->setSpacing( 0 );
	lyt->addWidget( titleLbl );
	lyt->addWidget( edit );

	setLayout( lyt );

	setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred ) );
};

bool DesQComboBox::isCompactMode() {

	return mCompactMode;
};

void DesQComboBox::setCompactMode( bool yes ) {

	mCompactMode = yes;

	qDeleteAll( children() );

	QBoxLayout *lyt;
	if ( yes )
		lyt = new QVBoxLayout();

	else
		lyt = new QHBoxLayout();

	lyt->addWidget( titleLbl );
	lyt->addWidget( edit );

	setLayout( lyt );
};

void DesQComboBox::addItem( QIcon icon, QString name, QVariant userData ) {

	edit->addItem( icon, name, userData );
};

void DesQComboBox::addItems( QStringList items ) {

	edit->addItems( items );
};

void DesQComboBox::removeItem( int index ) {

	edit->removeItem( index );
};

QString DesQComboBox::currentText() {

	return edit->currentText();
};

QIcon DesQComboBox::currentIcon() {

	return edit->currentData( Qt::DecorationRole ).value<QIcon>();
};

int DesQComboBox::currentIndex() {

	return edit->currentIndex();
};

QVariant DesQComboBox::currentData() {

	return edit->currentData( Qt::UserRole );
};

QString DesQComboBox::itemText( int index ) {

	return edit->itemText( index );
};

QIcon DesQComboBox::itemIcon( int index ) {

	return edit->itemIcon( index );
};

QVariant DesQComboBox::itemData( int index ) {

	return edit->itemData( index, Qt::UserRole );
};
